'' ILI9325 driver using pins P0-P11 for the display and P16-P23 for the touchscreen
'' James Moxham 2011

CON


  _clkmode      = xtal1 + pll16x                        ' use crystal x 16
  _xinfreq      = 5_000_000

  _SD_DO = 12
  _SD_CLK = 13
  _SD_DI = 14
  _SD_CS = 15
  _SD_WP = -1                                           ' -1 ifnot installed.
  _SD_CD = -1                                           ' -1 ifnot installed.

  _RTC_DAT = -1                                         ' -1 ifnot installed. For kye's sd driver
  _RTC_CLK = -1                                         ' -1 ifnot installed.

  _TV_Enabled = true                                    ' TV display on
  _TV_Pin = 16                                          ' group of 3 pins starting at pin 16 = 16,17,18

  Touch_Clock = 21              ' 23 old board                          ' Touch screen pins (shared with VGA and one, penirq, shared with TV)
  Touch_ChipSelect = 23         ' 22 old board
  Touch_DataIn = 22             ' 21 old board
  'Touch_Busy = 20              ' 20 old board
  Touch_DataOut = 20            ' 19 old board
  'Touch_Penirq = 18             ' 18 old board
  
  ' Memory map - aligned on 256 byte blocks for faster block reads
  ' Display from 0 - 153599
  ' Icons from 153600 to 169983 ' store background behind the icon
  ' A text buffer from 169984 to 202751  = 32k of text for text files
  ' A vt100 text buffer from 202752 to 206079 = 3328 bytes (80x40 and rounded up) 
  ' The current font from 206080 up

  RamText   = 169984
  VT100     = 202752
  FontTable = 206080 ' all fonttables and anything that uses the A command must start on 0,256,512,768 etc
                     ' .ifn font files have been rounded up so always fit in blocks of 256 (check the file size)
                     ' so read in the file size and then can put fonts one after the other 
                     ' on the other hand these files are big and may cause memory management problems
                     ' so may be better to only have one font at a time
                     ' note that there is kerning data as well with combinations of letters and the amount to shuffle

  ' VT100 display for text based programs
  VT100Cols = 80               ' eg 80x24
  VT100Rows = 40
  'VT100Size = VT100Rows * VT100Cols
  ' serial port speeds
  _baudRateSpeed = 115200 ' don't use Teraterm at 115200 and possibly even 38400. Use Shamcom/hyperterminal or vb.net
  _receiverPin = 31
  _transmitterPin = 30  

OBJ
 delay:         "timing"                                ' thanks to parallax
 fat:           "SD3.01_FATEngine.spin"                 ' thanks to Kye
 'text:          "tv_text"                               ' thanks to Chip
 str:           "ASCII0_STREngine.spin"                 ' thanks to Kye 
 spi:           "SPI_ASM"                               ' thanks to Beau
 ILI9325:       "ILI9325_Dracblade"                     ' J Moxham
 keyboard:      "keyboard"                              ' Chip Gracey
 sio:           "pcFullDuplexSerial2FC"                 ' Tim Moore and Juergen  Buchmueller

VAR
   byte LineOfText1[20] ' general purpose string buffer
   byte LineOfText2[20] ' general purpose string buffer
   byte FontHeight                ' size of the current loaded font (pixels = fontsize *.75)
   word BackFontColor           ' the background color in RRRRRGGG GGGBBBBB format
   word curx                    ' current cursor x position in pixels
   word cury                    ' current cursor y position in pixels
   byte Orientation                ' true for portrait, false for landscape
   word ScreenWidth             ' either 240 in portrait or 320 in landscape
   word ScreenHeight            ' 320 in portrait or 240 in landscape
   byte vt100curx               ' current cursor position in the vt100 screen
   byte vt100cury
   ' xmodem variables
   byte XmodemBuffer[132]
   byte validpacket
   word PacketNumber
   byte ErrorCounter
   byte OnesComplement
   word Checksum
   word Timeout
  
PUB Main 
    'text.start(_TV_Pin)                                 ' start the TV driver
    'text.clearscreen                                    ' clear the screen
    ILI9325.start_ram                                   ' start the external ram driver in a cog
    ILI9325.start_ILI9325                               ' start the ILI9325 display
    SetOrientation(true)                                '  in portrait (true) or landscape mode (false)   
    keyboard.startx(26, 27, %100, 4)                    'Start Keyboard Driver  pin,pin,num,repeatrate  
    'PrintStringCR(string("Testing for SD card"))       ' helpful message if card is out, better than just a blank screen  
    'Touchscreen                                        ' test the touchscreen
    ifnot( fat.FATEngineStart(_SD_DO, _SD_CLK, _SD_DI, _SD_CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1))   
      reboot
    fat.mountPartition(0)                               ' mount the sd card
    TouchStart                                          ' start the SPI touchscreen driver
    'EditLine
    sio.AddPort(0, _receiverPin, _transmitterPin, -1, -1, 0, 0, _baudRateSpeed) ' add a serial port for xmodem  
    sio.start                                           ' start the com port  note can't print anything to screen till sio.start unless only do to vga screen
    sio.rxflush(0)                                      ' make sure no data in the buffer

    Clearscreen(0)                                      ' clear the screen

    'Propfont ' ultra basic font display which does not need SD working as uses propeller font
    Bootmessage                                      ' bootmessage
    
    ' PUB ILIDebugStart 
    'ILIdebugString(string("Debug"))                     ' debug string and hex value
    ' ILIdebugHex($03A4EF27) 
    'repeat
    '   KeyboardTest                                     ' test the keyboard

    DrawBackgroundAndIcons 
    Touchscreen

PUB Xmodem | i ' for use with vb.net downloads as need to send the command xmodem r myfile.ext and then the data
' runs when either touch the button, or an "X" comes in the serial port
    SetOrientation(true)                              ' portrait
    NewFont(string("Par16.ifn"))                      ' Parallax 8x16 font (16 point) with no smoothing (small fonts better not smoothed)    
    ILISetcursor(0,0)                                  ' sets curx and cury to zero
    ClearFontBackground(0,0,screenwidth,screenheight)
    ILIPrintstringcrlf(string("Xmodem file receive")) ' print string and crlf
    str.str(@lineoftext1,_baudratespeed)
    ILIPrintstring(string("Baud rate = "))
    ILIPrintstringcrlf(@lineoftext1)
    ILIPrintstringcrlf(string("Wait for XMODEM R MYFILE.EXT"))   
    sio.rxflush(0)                                      ' make sure no data in the buffer
    result:=0
    str.builtstring(true)                               ' resets the string ready to build a new one
    repeat
        if sio.rxavail(0)
          result :=sio.rx(0)
          if((result => "a") and (result =< "z"))
             result := (result - 32)                    ' convert to upper case
          if result >31
            sio.tx(0,result)                            ' echo back if >31
            curx := ILIChar(result,curx,cury)           ' print the incoming control XMODEM R MYFILE.EXT <cr>
            str.buildstring(result)      
    until result == 13                                  ' build command string until get "enter"
    curx :=0
    cury += fontheight
    result := str.len(str.builtstring(false))           ' length of the string "XMODEM R MYFILE.TXT" 
    str.mid(str.builtstring(false),@lineoftext1,10,result-9)

    'xmodem receive copied from kyedos xmodem program

    ILIPrintstring(string("Receiving filename: "))
    ILIPrintstringcrlf(@lineoftext1)
    ILIPrintstring(string("Send NAK and waiting for reply")) ' only print on local display, not to serial port
    i:=0                                                ' set counter to 0
    sio.rxflush(0)                                      ' clear the receive buffer
     repeat while (i < 15 and sio.rxavail(0)==0)        ' send 15 NAKs don't make the delay too short
       curx := ILIChar(".",curx,cury)                   ' print a . on local display
       sio.tx(0,21)                                     ' send a NAK to start things off
       delay.pause1ms(1000)                             ' wait 1 second 
       i++                                              ' add 1 to i
    if sio.rxavail(0)
       'vt100.str(string("open file"))
       result :=\fat.newfile(@lineoftext1)                   ' does this file already exist? The \ says to keep going even if there is an error
       if(fat.partitionError == fat#Entry_Already_Exist)  ' file already exists
            'vt100.str(string("delete entry"))
            result := \fat.deleteEntry(@lineoftext1)    ' delete the old file
            result := \fat.newfile(@lineoftext1) ' recreate the file
            ILIPrintstring(string(" Deleting previous file "))
            ILIPrintstring(string(" Creating file "))        ' print creating new file message
       fat.openFile(@lineoftext1,"W")                      ' open file   
       ILIPrintstringcrlf(string("Starting file transfer"))
       delay.pause1ms(100)                              ' short delay makes "1st packet errors" less likely                        
       errorcounter:=0                                  ' reset error counter
       packetnumber:=1                                   ' packet number 1
      repeat
         validpacket :=255                              ' equals true, set to 0 if any errors further down
         timeout:=0
         xmodembuffer[0] := 0                           ' definitely clear for startup                        
         repeat while timeout < 100                      ' timeout if no byte arrives must be <255
           if sio.rxavail(0)                              ' is there a byte?
             XmodemBuffer[0]:=sio.rx(0)                         ' read in one byte
             timeout:=65000                               ' finish the timeout
           else
             delay.pause1ms(1)                          ' wait 1 ms
             timeout++                                  ' add 1 to timeout
        if XmodemBuffer[0] <> 4                         ' finish byte
          if XmodemBuffer[0] <> 1                       ' not 1 so packet corrupt or timed out (0)
            validpacket:=0                              ' invalid packet
            errorcounter++                              ' add one to error counter
            curx := ILIchar("x",curx,cury)              ' a common error on 1st packet so print something short
            'vt100.dec(XmodemBuffer[0])                  ' print out the first item in the buffer
            sendNAK                                     ' send a NAK/flush buffer and print

          else                                          ' 1= a valid start byte for a packet
            i:=1
            timeout:=0                                  ' if times out then packet will be corrupt
            repeat while i<132                         ' includes a timeout if half packet missing
              if sio.rxavail(0)                         ' if a byte is available
                XmodemBuffer[i]:=sio.rx(0)                    ' read it in
                i++                                       ' and increment
              else
                delay.pause1ms(10)                        ' otherwise, pause a little
                timeout++                                 ' and keep track of the pauses
                if timeout>100                            ' and give up after 1 second total
                  i:=133                                 ' force end if there is a timeout
                  errorcounter++                        ' and add one to error counter
                  curx := ILIchar("t",curx,cury)        ' an error

            'repeat i from 0 to 131                      ' debugging
             'vt100.hex(xmodembuffer[i],2)
             'vt100.out(",")
                    
            packetnumber &= 255                         ' bitwise AND with 255, quicker than repeatedly subtracting 256      
            onescomplement:=255-packetnumber            ' calculate the ones complement
            if packetnumber <> XmodemBuffer[1]
              validpacket:=0                            ' not a valid packet
              errorcounter++
              curx := ILIchar("p",curx,cury)
            if XmodemBuffer[2] <> onescomplement            ' ones complement not correct
              validpacket:=0
              errorcounter++
              curx := ILIchar("o",curx,cury)                       
            checksum:=0                                 ' calculate the checksum
            repeat i from 3 to 130
               checksum:=checksum+XmodemBuffer[i]

            checksum &= 255                             ' bitwise AND with 255, same as repeatedly subtracting 255
            'repeat while checksum>255                   ' round to under 256
                'checksum:=checksum-256
            if checksum <> XmodemBuffer[131]                   ' checksum not valid
              validpacket:=0
              errorcounter++
              curx := ILIchar("c",curx,cury)
            if validpacket == 255                         ' packet is valid so save it
              fat.writeData(@XmodemBuffer+3,128)
              packetnumber++                             ' increment packet number
              sio.tx(0,6)                                ' send a ACK and wait for the next packet
              curx := ILIchar(".",curx,cury)                             ' print a . on the screen for each packet
            else
              sendNAK
              curx := ILIchar("N",curx,cury)
      until errorcounter>9 or xmodembuffer[0]==4 ' too many errors, or 04=the finish byte
       sio.tx(0,6)
      fat.CloseFile
      if errorcounter==0
        ILIPrintstringcrlf(string("No errors"))
      if errorcounter>0 and errorcounter<3
        ILIPrintstringcrlf(string("File OK, some corrected errors"))
      if errorcounter>2
        repeat i from 1 to 10
          ILIPrintstring(string("Cancel "))
          sio.tx(0,18)                                  ' Send Cancel 10 times
      ILIPrintstringcrlf(string("Rebooting"))
      delay.pause1ms(500) ' so can see messages before reboots  


PRI SendNAK
    sio.rxflush(0)
    sio.tx(0,21)                               ' send a NAK as the packet was corrupted
    errorcounter++
    ILIPrintstring(string(" NAK "))       ' increment the error counter
    delay.pause1ms(500)  




    
PUB SetOrientation(e) ' change orientation true = portrait. Changes global variable 'orientation' and also screen width and height
   orientation := e
   ILI9325.ChangeOrientation(orientation)
   if orientation
     screenwidth :=239
     screenheight :=319
   else
     screenwidth :=319
     screenheight :=239
{
PUB Propfont  ' use propeller font in the rom for bootup messages when debugging sd cards
    curx := 0                               ' set cursor to top left of screen (common variable)
    cury := 0
    Propfont_string(string("Loading SD"))   ' string to send
    Propfont_string(string("Loading Font"))

PUB Propfont_string(stringptr)                          'print at curx,cury
  repeat strsize(stringptr)
    ILI9325.draw(curx,cury,curx+15,cury+31)             ' location to start drawing    
    Propfont_out(byte[stringptr++])   
    curx +=16
  curx := 0
  cury += 32                                            ' new line at end of string

PUB Propfont_out(ascii) | address,pixels
    address := $8000 + (ascii >> 1) << 7                ' get rom address
    repeat 32                                           ' 32 rows per character, split in two parts
      pixels := long[address]                           ' get rom font data
      pixels := pixels >> (ascii & 1)                    ' shift for odd characters
      repeat 16 ' 16 columns
        if pixels & 1
          ILI9325.pixel(%00000111_11100000)             ' foreground color RRRRRGGG_GGGBBBBB
        else
          ILI9325.pixel(%00000000_00000000)             ' background color 
        pixels := pixels >> 2                           ' alternate pixels interleaved so shift 2
      address += 4                                      'increment address by 4 bytes
}

     

PUB KeyboardTest
  ILIDebugStart                                         ' load the debug font
      result:=0
      repeat
        if keyboard.gotkey <>0                          ' scan keyboard
          result:=keyboard.getkey
      until result <>0                                  ' until something comes in 
    if result==200
      result :=8                ' backspace keycode is 200 but convert to 8
    ILIdebugHex(result)

PUB EditLine | filesize,pointer,c  ' edit a line of text. Flashing cursor. Insert on/off. Arrow keys to move cursor. Use real keyboard
     ILISetcursor(0,0)                                  ' sets curx and cury to zero
     SetOrientation(false)                              ' set to landscape
     NewFont(string("4x6.ifn"))                          ' load 4x6 font
     ClearFontBackground(0,0,screenwidth,screenheight)
     vt100clear                                         ' need to clear to spaces as errors with ascii >126
     ' tiny 6x4 font from http://robey.lag.net/2010/01/23/tiny-monospace-font.html
     filesize := Loadfile(string("ws.txt"))                                           ' load in the text to external ram
     pointer := RamText
     repeat 3200
       c := ILI9325.readrambyte(pointer)
       vt100out(c)                         ' just print it
       pointer ++
     vt100redraw     ' could add the 4x6Y yellow font in vt100redraw - need a color buffer 80x40 for foreground and another for background
     repeat

PUB VT100Clear | i
   longfill(@sdbuffer,$20202020,64)                     ' fill with spaces
   i := vt100 ' location of vt100 buffer
   repeat 13                                         ' 13*256 is the size of the buffer (80*40 rounded up)
     ILI9325.docmd("W",@sdbuffer,vt100+ i*256,256)
     i++
   vt100curx := 0
   vt100cury := 0

PUB VT100Locate(x,y)
    vt100curx := x                                      ' global variables
    vt100cury := y

PUB VT100out(c)
    if c>31 and c <127  ' need to add in cr and lf and backspace
      VT100Store(c)
      vt100curx += 1
      if vt100curx > (vt100cols -1)    
        vt100curx := 0
        vt100cury +=1    

PUB VT100Store(c) ' store character c at vt100curx,vt100cury where 0,0 is top left corner
    ILI9325.docmd("W",@c,vt100+ (vt100cury * vt100cols) + vt100curx,1)

PUB VT100Redraw | x,y' redraw the entire VT100 screen. Assumes font is loaded
    ClearFontBackground(0,0,screenwidth,screenheight)
    ILIsetcursor(0,0)
    repeat y from 0 to (vt100rows - 1)
      ILI9325.docmd("R",@sdbuffer,vt100 + (y * vt100cols),80) ' read one row which will always be =< 80
      curx := 0
      repeat x from 0 to (vt100cols -1)
        curx := ILIChar(sdbuffer[x],curx,cury)
      cury += fontheight

PUB VT100String(stringptr)
  repeat strsize(stringptr)
      VT100store(byte[stringptr++])          ' post increment the stringptr. Need to handle writing off the end
      vt100curx++

PUB Bootmessage
    Clearscreen(0)
    ILIloadfont(string("JDLCD.ifn"))                     ' LED dot matrix type font green on black         
    ILISetcursor(0,0)                                   ' set cursor to this location
    ClearFontBackground(0,0,screenwidth,FontHeight)                 ' clear an area the same color as the font background       
    ILIprintstring(string(">Booting"))
     

PUB Calculator | xval,yval,address,ascii,operand
    FullScreenRaw(string("calcpic.raw"))                                      ' draw the calculator
    ILIloadfont(string("DSdig.ifn"))
    str.builtstring(true)       ' clear the working string 
    repeat
      yval := TouchValue >> 8                               ' decode yval 0-100%               
      xval := TouchValue & 255                              ' decode xval 0-100%
      if (xval <> 255)  and (yval <> 255)                    ' valid keypress
        address :=0
        case (yval*32)/10                                    ' convert 0-100 to 0-320 as easier to work with pixels off the picture
          60..101: address :=0                               ' row of a 4x6 matrix
          104..140: address :=4
          146..183: address :=8
          187..222: address :=12
          228..265: address :=16
          271..307: address :=20
        case (xval*24)/10                                   ' column convert percent to 0-239
          0..66: address += 0
          67..119: address +=1
          120..172: address +=2
          173..239: address +=3
        ascii := lookupz(address: "X","Q","Q","Q","C",8,"/","*","7","8","9","-","4","5","6","+","1","2","3","=","0","0",".","=")
        case ascii
          "X"             : return                                              ' close the calculator
          "C"             : str.builtstring(true)                               ' clear the current string
                            CalcClear
          "0".."9",8      : str.buildstring(ascii)                              ' build the string. Can also handle backspace
                            CalcClear                            
                            ILIprintstring(str.builtstring(false))              ' reprint the string each new char in case a backspace 
          "+","-","*","/" : str.copy(str.builtstring(true),@lineoftext1)        ' move the string to lineoftext1 and reset
                            operand := ascii
          "=" :             xval := str.decimaltointeger(@lineoftext1)          ' first operand. recycle xval and yval
                            yval := str.decimaltointeger(str.builtstring(true)) ' second operand
                            case operand
                                  "+" : address := xval + yval                  ' add
                                  "-" : address := xval - yval                  ' subtract
                                  "*" : address := xval * yval                  ' multiply
                                  "/" : address := xval / yval                  ' integer division
                            str.str(@lineoftext2,address)                       ' convert answer to string
                            str.trimstring(@lineoftext2) ' if negative, leave the -, if positive remove the leading space
                            CalcClear                                           ' clear the display ready to print the answer
                            ILIprintstring(@lineoftext2)                        ' print the answer
        delay.pause1ms(250)' only display once so delay half a second to "debounce"

PRI CalcClear
     ClearFontBackground(25,21,220,48)       ' clear an area the same color as the font background        
     ILISetcursor(25,15)  

PUB DrawBackgroundAndIcons
    FullScreenRaw(string("beach.raw"))                                       ' load picture 240 wide, 320 high
    DisplayIcons

PUB PhotoAlbum | picturenumber
   picturenumber := 1
   repeat until picturenumber > 5
     if picturenumber == 1
        FullScreenRaw(string("cat.raw"))         ' draw a portrait picture
     if picturenumber ==2
        SetOrientation(false)                           ' draw a landscape picture
        FullScreenRaw(string("wallaby.raw"))
        SetOrientation(true)     
     if picturenumber == 3
        FullScreenRaw(string("kluger.raw"))
     if picturenumber == 4
        FullScreenRaw(string("elle.raw"))    
     if picturenumber == 5
        FullScreenRaw(string("balloon.raw"))
     result := Touchswish
     if result > 0
       picturenumber --
     else
       picturenumber ++

PUB ILILoadFont(stringptr) | filesize,i                ' read a .ifn file premade in angelcode bmfont and then the vb.net program
    ' pass file name in stringptr
    ' the variable "i" must start on 0,256,512,768 etc for the block move to work
    i := fonttable                                      ' store at the constant 'fonttable' location
    fat.openfile(stringptr,"R")                      ' open an image. Size is in the first long and has been rounded up to the next 256 byte block
    filesize := fat.filesize                            ' read the file size. This file has been rounded up to the next 256 byte block
    repeat (filesize >> 8)                              ' read in this number of 256 byte blocks
      fat.readdata(@sdbuffer,256)                       ' get 256 bytes from sd card  
      ILI9325.docmd("A",@sdbuffer,i,256)      ' A must start on 0,256,512,768 etc move data from hub to external ram 256 bytes at a time    
      i += 256                             ' add 256
    fat.closefile
    FontHeight := ILI9325.readrambyte(fonttable+251)    ' extract font size
    ' extract the background color so can draw the background behind the text
    repeat i from 0 to 2
      sdbuffer[i] := ILI9325.readrambyte(fonttable + 35+i)   ' read the 3 RGB bytes
    ILI9325.docmd("E",@sdbuffer,@buffer2,1)             ' convert to 2 byte .ili format, len is number of pixels        
    BackFontColor := word[@buffer2][0]
    result := filesize    

      
PUB ILIChar(ascii,x,y) | jump,size,width,height,ramaddress,xoffset,yoffset,xadvance   ' read out a character from a .ifn file stored in ram
  ' pass Fonttable = global
  ' moves to next line if off the end
    jump := ILI9325.ReadRamLong((ascii << 2) + Fonttable + 256)' jump location = ascii *4 plus fonttable + 256 
    jump += fonttable                                   ' add the fonttable location
    size := ILI9325.ReadRamLong(jump)                   ' size precalculated = width x height x 2
    xadvance :=  ILI9325.ReadRamLong(jump+20)     '     ' amount to move to next character
    if size > 0 and ascii <> 32                                           ' no need to print anything if size is zero or read more font data
      width :=   ILI9325.ReadRamLong(jump+4)                ' width
      height :=  ILI9325.ReadRamLong(jump+8)            ' height
      xoffset := ILI9325.ReadRamLong(jump+12)           ' xoffset to move
      yoffset := ILI9325.ReadRamLong(jump+16)           ' yoffset to move
      if (x+width -1 + xoffset) > screenwidth
        ILIcrlf ' do a new line if it won't fit
        x :=curx
        y :=cury
      ILI9325.draw(x+xoffset,y+yoffset,x+width-1+xoffset,y+height-1+yoffset)      ' draw on screen
      ILI9325.docmd("C",0,jump+32,size)              ' move bytes from ram out to the display  
    return x + xadvance


PUB ILIPrintstring(stringptr)                 'print at curx,cury
'' Print a zero-terminated string
  repeat strsize(stringptr)
    curx := ILIChar(byte[stringptr++],curx,cury)                           


PUB ILISetCursor(x,y)            
    curx := x
    cury := y

PUB ILIPrintstringcrlf(stringptr) ' print string and crlf
    ILIprintstring(stringptr)
    ILIcrlf

PUB ILIDebugStart
    ILILoadFont(string("Arial40.ifn"))
    ILISetcursor(0,279)                              ' set cursor to this location    

PUB ILIdebughex(number) ' print a debug value on the screen in hex. Run start above to set up
    ClearFontBackground(0,279,239,279+FontHeight)                 ' clear an area the same color as the font background       
    ILISetcursor(0,279)
    ILIPrintstring(string("0x"))
    str.copy(str.integertohexadecimal(number,8),@lineoftext1)
    ILIPrintstring(@lineoftext1)

PUB ILIdebugString(stringptr) ' print a debug string on the screen
    ILILoadFont(string("Arial40.ifn"))
    ILISetcursor(0,239)                              ' set cursor to this location
    ClearFontBackground(0,239,239,239+FontHeight)                 ' clear an area the same color as the font background       
    ILIprintstring(stringptr)

'PUB ILIDecimal(number)
'      str.str(@lineoftext1,number) ' decimal to string with no leading zeros
'      ILIPrintstring(@lineoftext1)

      'str.copy(str.integertodecimal(number,10),@lineoftext1) ' or add lots of leading zeros. Your choice
      'ILIstr(@lineoftext1) 

PUB ILIcrlf
    curx := 0
    cury += FontHeight
    if cury > screenheight
      ClearFontBackground(0,0,screenwidth,screenheight) 
      cury := 0                 ' if off the bottom of the screen then clear screen start at the top (VT00 does scrolling)
    ' don't clear background here, do in calling routine
    'ClearFontBackground(0,cury,screenwidth,cury + FontHeight) ' clear background line to the background color ready to draw          

PUB FullScreenRaw(stringptr) | ramaddress                         ' convert 3 byte .raw file to 2 byte .ili file
    fat.openfile(stringptr,"R")                ' open an image
    ramaddress := 0
    repeat 600
      fat.readdata(@sdbuffer,384)                       ' 128 pixels, 3 bytes per pixel
      ILI9325.docmd("E",@sdbuffer,@buffer2,128)         ' convert to 2 byte .ili format, len is number of pixels
      ILI9325.docmd("A",@buffer2,ramaddress,256)        ' A must start on 0,256,512,768 etc move bytes to external ram
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile
    RamToDisplay                              ' move bytes from ram out to display

PUB DrawKeyboard | ramaddress
    ' reload the font as gets overwritten by icons
    ILILoadfont(string("mssans12.ifn"))
    fat.openfile(string("keyboard.raw"),"R")                ' open the keyboard picture   
    ramaddress := 0
    repeat 360
      fat.readdata(@sdbuffer,384)                       ' 128 pixels, 3 bytes per pixel
      ILI9325.docmd("E",@sdbuffer,@buffer2,128)         ' convert to 2 byte .ili format, len is number of pixels
      ILI9325.docmd("A",@buffer2,ramaddress,256)        ' A must start on 0,256,512,768 etc move 256 bytes to external ram (must be 256)
      ramaddress += 256                                 ' add 256
    fat.closefile
    ILI9325.draw(0,128,239,319)                           ' 240x192 for the keyboard
    ILI9325.docmd("C",0,0,92160)                       ' move bytes from ram out to the display
    
PUB Touchkeyboard  | yval, xval,address,ascii,index,capslock, backspace
    ' from the top of the keyboard picture - add 128 for screen pixel 0-319
    ' y=33-61 is the numbers ( 161 - 189 which is 50% to 59%) 
    ' y = 64-93 is qwerty    (192 to 221 which is 60 to 69%)
    ' y = 96-124 is asdf      (224 to 252 which is 70-79%)
    ' y= 128-157 is zxcv      (256 to 285 which is 80 to 89%)
    ' y=160-189 is space
    capslock := 0 ' default to off
    backspace := 0 'default to off
    ILISetcursor(10,137)                              ' set cursor to this location    
    repeat
      yval := TouchValue >> 8                               ' decode yval 0-100%               
      xval := TouchValue & 255                              ' decode xval 0-100%
      if (xval <> 255)  and (yval <> 255)               ' a valid value

        if (yval => 50) and (yval =< 59)                ' number row 0-100%
          address := xval / 10                              ' 10 values in row = 0 to 9
          'ILIdecimal(address)                                     ' test debugging on the ILI screen.

        if (yval => 60) and (yval =< 69)
          address := 10 + (xval / 10)                   ' address = 10 to 19    QWERTYUIOP

        if (yval => 70) and (yval =< 79)
          address := 20 + ((xval - 5) / 10)             ' address = 20-28 ASDFGHJKL shift left slightly as this row is offset half a key

        if (yval => 80) and (yval =< 89)                ' third row
          if (xval < 15)
            !capslock                                   ' swap 0,FFFFFFFF capslock
            address := 255                              ' so no character printed
            'printstringcr(string("Caps lock"))
          if (xval =>15) and (xval =< 85)
            address := 29 + ((xval - 15) / 10)          ' ZXCVBNM
          if xval > 85
            backspace := 255                             ' backspace - need to be storing in a string and delete the last character and reprint the string                             

        if yval => 90                                   ' fourth row
           if xval <25
           ' not sure what to do with this 123 button, maybe all the !@#$%^ characters above the numbers
           if (xval => 25) and (xval =< 75)
             address := 36                              ' space  button
           if xval > 75                                 ' enter button
           ' exit here and return a string
           return
           
        if address < 255                                ' not a backspace or an enter or a capslock
          ascii := lookupz(address: "1","2","3","4","5","6","7","8","9","0","Q","W","E","R","T","Y","U","I","O","P","A","S","D","F","G","H","J","K","L","Z","X","C","V","B","N","M"," ")
          if (capslock <> 0) and (ascii => 65) and (ascii =< 90) 
            ascii += 32                                 ' lower case
          curx := ILIChar(ascii,curx,cury)
             
      delay.pause1ms(250)' only display once so delay half a second to "debounce"   

PUB RamToDisplay ' dumps preloaded ram to display starting address 0
    'Clearscreen                                        ' leave out clearscreen
    ILI9325.draw(0,0,screenwidth,screenheight)                           ' set up the area of the screen to draw in
    ILI9325.docmd("C",0,0,153600)                       ' move bytes from ram out to the display

PUB Movie                                              ' maybe need to speed up the draw routine as this is in spin
    clearscreen(0)
    fat.openfile(string("taz.ilm"),"R")             ' also can skip alt frames, and maybe make the sd buffer bigger
    repeat 338 ' number of frames
      ILI9325.draw(60,115,179,204)                           ' this can be sped up as this is in spin
      repeat 45                                         ' 120*90*2 / 480
        fat.readdata(@sdbuffer,480)                     ' read data into hub
        ILI9325.docmd("D",@sdbuffer,0,480)              ' send data out to the display (could possibly do some in parallel, though the ILI display may be faster than the sd card)
    fat.closefile
PUB Clearscreen(color) 
    bytefill(@sdbuffer,color,512)
    ILI9325.draw(0,0,screenwidth,screenheight)                           ' set up the area of the screen to draw in   
    repeat 300
      ILI9325.docmd("D",@sdbuffer,0,512)                       ' move bytes from ram out to the display

PUB ClearFontBackground(x1,y1,x2,y2) | i,size,remainder ' clears an area of the screen to the current font background color
    size := (x2-x1+1) * (y2 - y1 +1) ' number of pixels
    remainder := (size & 255) << 1                      ' if not a whole number of 256 pixels
    repeat i from 0 to 510 step 2                         ' move the background font colour to the buffer
      sdbuffer[i] := BackFontColor & 255                  ' and replicate for 256 pixels
      sdbuffer[i+1] := BackFontColor >> 8
    ILI9325.draw(x1,y1,x2,y2)                           ' set up the area of the screen to draw in
    repeat size >> 8                                    ' 512 byte blocks
      ILI9325.docmd("D",@sdbuffer,0,512)                ' 2 = 2 bytes which = one pixel
    if remainder <> 0  
      ILI9325.docmd("D",@sdbuffer,0,remainder)           ' do the remainder          
 
PUB TransparentIcon(stringptr,x,y) | sizex,sizey,backgroundaddress,ramaddress ' draw icon name in LineOfText1
    x := x * 60 ' pass x as 0,1,2,3 from touchscreen convert to a pixel coordinate
    y := y * 64 ' pass y as 0,1,2,3,4 from touchscreen 320/6 = 64
    sizex := 59
    sizey := 60
    ILI9325.draw(x,y,x+sizex-1,y+sizey-1)                           ' set up the area of the screen to draw in
     ' background is stored in memory 0 to 153599
     ' background behind the icon temp store =64*64*4 = 16384 bytes 153600 to 169984 so contiguous bytes
    backgroundaddress := (((y*240)+x) << 1)              ' calculate the correct location in background ram
    ramaddress := 153600
    repeat 30
      ILI9325.docmd("R",@sdbuffer,backgroundaddress,(sizex << 2)) ' get 59*2 *2 bytes from background. Instead of 60 lines with everything multiplied by 2, do 30 lines and multiply by 4
      ILI9325.docmd("W",@sdbuffer,ramaddress,(sizex << 2)) ' save to a temp buffer 
      backgroundaddress += (240 << 2)                          ' next row = 240 pixels *2  *2
      ramaddress += (sizex << 2)                        ' 59*2  *2
    ramaddress := 169984 ' location to store icon
    'PrintStringCR(string("Load"))            ' message on the TV screen
    fat.openfile(stringptr,"R")              ' open an icon image (.icn extension = RGBA)
    repeat sizey
      fat.readdata(@sdbuffer,(sizex << 2))                 ' sizex pixels * 4 bytes per pixel so multiply by 4
      ILI9325.docmd("W",@sdbuffer,ramaddress,(sizex << 2)) ' now move data from hub to external ram
      ramaddress += (sizex << 2)                        ' 4 bytes per pixel so multiply by 4   
    fat.closefile   
    'PrintStringCR(string("Closing file"))
    ILI9325.docmd("F",153600,169984,(sizex*sizey))  ' background,icon,number of pixels

PUB TouchStart
  OUTA[Touch_ChipSelect] := 0                           ' enable the touch screen
  SPI.start(3,0)                                        ' delay,state

PUB TouchX
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1101_0000)  ' reads x from  500 to 3700  (off < 500 )
      result := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)

PUB TouchY      
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1001_0000)  ' reads y from 400 to 3800 (off > 3800 ) 
      result := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)    

PUB TouchValue | xval,yval,x,y ' returns % values 0-100% (or 255 for no touch)
'  Touch_Clock = 21                                       ' clock
'  Touch_ChipSelect = 23                                  ' chip select needs to be low to enable the touchscreen
'  Touch_DataIn = 22                                      ' data into the chip = an output from the prop
'  Touch_Busy = 20                                        ' busy ? not needed - just returns /cs
'  Touch_DataOut = 20                                     ' data out of the chip = an input to the prop
      xval := TouchX
      yval := TouchY
      if (xval => 500) and (yval =< 3800)                       ' both have to be valid for a touch
        delay.pause1ms(2)                               ' tiny delay then read again as the first reads may not have the correct value
        xval := TouchX
        yval := TouchY
        if (xval => 500) and (yval =< 3800)                 ' and if valid again, now more likely this value is correct
            x := xval - 440                                    ' scale 0-3360
            x /=  32                                     ' scale 0-100
            x <#= 100                                    ' max 100
            x #>=0                                       ' min 0
            x := 100 - x                                 ' so left edge on left
            y := yval -420                               ' scale 0-3180
            y /= 32                                      ' scale 0-100
            y <#= 100                                    ' max 100
            y #>=0                                       ' min 0
            y := 100-y                                   ' so 0 is top left     
            result := (y <<8) | x                       ' return 00000000_00000000_YYYYYYYY_XXXXXXXX   
      if (xval < 500) or (xval >3800)
        result := $0000FFFF                             ' invalid number(s)


PUB TouchSwish | xval,oldxval,difference,average ' returns result = +ve or -ve depending on swish direction     ' did finger move left or right?
      oldxval := 255' for startup
      average := 0
      repeat until (average > 30) or (average < -30)
        ' possibly add a decay function here for average
        delay.pause1ms(50)
        xval := TouchValue & 255                        ' decode xval 0-100%
        if xval == 255                                  ' not touched
          oldxval := 255                                ' reset oldxval as well
        else
          if oldxval <>255                                ' discard the first reading as the screen is touched
            difference := xval - oldxval
            'printdecimal(xval)
            'printdecimal(difference)
            average := average + difference             ' 
            'printdecimal(average)
            'crlf
          oldxval := xval                                 ' store for next time
      result := average                                 ' return negative or positive value

          

PUB DisplayIcons
    TransparentIcon(string("photos.icn"),1,1) ' display icon
    TransparentIcon(string("key.icn"),3,0) ' display icon
    TransparentIcon(string("videos.icn"),2,4) ' display icon
    TransparentIcon(string("prop.icn"),0,4) ' display icon
    TransparentIcon(string("land.icn"),0,2) ' display icon
    TransparentIcon(string("calc.icn"),1,4)        ' display 
    TransparentIcon(string("80x40.icn"),3,4)        ' display
    TransparentIcon(string("Safari.icn"),0,3)        ' display
    TransparentIcon(string("xmodem.icn"),1,3)        ' display xmodem

PUB TouchScreen  | yval, xval,x,y,pointer
    repeat
       if sio.rxavail(0)             ' any X on the serial port, if so, then run xmodem
          if sio.rx(0) == "X"
            xmodem
            DrawBackgroundAndIcons
      yval := TouchValue >> 8                               ' decode yval 0-100%               
      xval := TouchValue & 255                              ' decode xval 0-100%
      if (xval <> 255)  and (yval <> 255)
        'printdecimal(xval)
        'printdecimal(yval)
        x := xval / 25                                  ' scale 0,1,2,3
        y := yval / 20                                  ' 0,1,2,3,4
        pointer := y*10+x       ' so can use a case statement
        case pointer
           0:                   'x=0,y=0
           1:                   ' x=1,y=0
           2:
           3: DrawKeyboard                 ' popup keyboard
              Touchkeyboard
          10:
          11: PhotoAlbum
          12:       
          13:
          20: SetOrientation(false) ' landscape
              Lorem
              SetOrientation(true) ' portrait
              Lorem
          21:
          22:
          23:
          30: html                                     ' html viewer
          31: xmodem                                    ' file transfers
          32:
          33:
          40: Proptool                                  ' run the proptool simulator
          41: Calculator
          42: Movie                                     ' play movie if hit the movie icon
          43: Editline                                  ' vt100 experiments

        DrawBackgroundAndIcons       
        delay.pause1ms(250)' only display once so delay half a second to "debounce"

PUB Proptool | filesize,pointer,c
     'str.copy(string("demo.txt"),@lineoftext1)          ' new faster font file - one file, designed to be read as fast as possible
     filesize := Loadfile(string("demo.txt"))                                          ' load in the text to external ram
     ILISetcursor(0,0)                                  ' sets curx and cury to zero
     NewFont(string("ParBY16.ifn"))                     ' brown on yellow
     pointer := RamText
     repeat filesize
       c := ILI9325.readrambyte(pointer)
       if c == 13
          ILIcrlf                                       ' new line, curx to zero and cury down by one font height
       'case pointer - Ramtext
         '$28:  NewFont(string("ParbBY16.ifn"))          ' bold black on yellow
         '$8B:  NewFont(string("ParGY16.ifn"))           ' green on yellow
         '$96:  NewFont(string("ParbBR16.ifn"))          ' bold black on red
         '$9B:  NewFont(string("ParBR16.ifn"))           ' black on red
         '$12E: NewFont(string("ParbBO16.ifn"))          ' bold black on orange
         '$133: NewFont(string("ParBO16.ifn"))           ' black on orange
         '$14D: NewFont(string("ParOO16.ifn"))           ' orange on orange
         '$16E: NewFont(string("ParBO16.ifn"))           ' black on orange
         '$186: NewFont(string("ParOO16.ifn"))           ' orange on orange
         '$1A7: NewFont(string("ParBO16.ifn"))           ' black on orange
         '$1BF: NewFont(string("ParOO16.ifn"))           ' orange on orange
         ''$1DD: NewFont(string("ParBO16.ifn"))          ' black on orange
         '$1E2: NewFont(string("ParbBC16.ifn"))          ' bold black on cyan
         '$1E5: NewFont(string("ParBC16.ifn"))           ' black on cyan
         '$2A0: NewFont(string("ParRC16.ifn"))           ' red on cyan     
         '$307: NewFont(string("ParCC16.ifn"))           ' cyan on cyan for comments
         '$313: NewFont(string("ParBC16.ifn"))           ' black on cyan
         '$3C2: NewFont(string("ParbBC16.ifn"))          ' bold black on cyan
         
       if (c=>32) and (c=< 126)
         curx := ILIChar(c,curx,cury)                   ' only print if a valid ascii character
       pointer ++

PUB Lorem | filesize,pointer,c     
     Clearscreen(255)
     filesize := Loadfile(string("Lorem2.txt"))                                           ' load in the text to external ram
     ILISetcursor(0,0)                                  ' sets curx and cury to zero
     NewFont(string("Times.ifn"))                     'load font
     pointer := RamText
     repeat 240
       c := ILI9325.readrambyte(pointer)
       if c == 13
          ILIcrlf                                       ' new line, curx to zero and cury down by one font height

       if (c=>32) and (c=< 126)
         curx := ILIChar(c,curx,cury)                   ' only print if a valid ascii character
       pointer ++

PUB Browser | ramaddress
    fat.openfile(string("browser.raw"),"R")                ' open an image
    ramaddress := 0
    repeat 80
      fat.readdata(@sdbuffer,384)                       ' 128 pixels, 3 bytes per pixel
      ILI9325.docmd("E",@sdbuffer,@buffer2,128)         ' convert to 2 byte .ili format, len is number of pixels
      ILI9325.docmd("A",@buffer2,ramaddress,256)        ' A must start on 0,256,512,768 etc move bytes to external ram
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile
    ILI9325.draw(0,0,screenwidth,32)                           ' set up the area of the screen to draw in
    ILI9325.docmd("C",0,0,20480)                       ' move bytes from ram out to the display

PUB HTML | filesize,pointer,c,ramaddress,blocks
    SetOrientation(false)                               ' landscape
    Clearscreen(255)                                    ' white background
    Browser                                             ' browser header at the top of the screen
    ILISetcursor(0,33)                                   ' local cursor
    filesize := Loadfile(string("html.txt"))            ' load into ram
    pointer := ramtext                                  ' pointer to ram
    ILILoadfont(string("Times16.ifn"))                      ' default font
    repeat until c == 254
      if c <> "<"
        c := ILI9325.readrambyte(pointer++)             ' don't read in if already got a "<"
      if c == "<"                                      ' <
          c := ILI9325.readrambyte(pointer++)            ' get next character
          if c == "p"                                   ' p for paragraph
            c := ILI9325.readrambyte(pointer++)           ' read in ">" and discard
            ILIcrlf                                     ' new paragraph
            c := 255                                      ' to exit
          if c == "b"                                   ' could be big or just b for bold
            c := ILI9325.readrambyte(pointer++)            ' get next character
            if c == ">"                                 ' it was b for bold
              ILILoadfont(string("Times16b.ifn"))
              c := 255                                  ' exit
            if c == "i"                                 ' must be big
               ILILoadfont(string("Times20.ifn"))
               repeat 2                                 ' read in rest of tag
                 ILI9325.readrambyte(pointer++)
               c :=255                               
            if c == "o"                                 ' body so discard
              repeat 3
                ILI9325.readrambyte(pointer++)
              c:= 255
            if c == "r"                                 ' must be <br>
              ILI9325.readrambyte(pointer++)            ' new line
              ilicrlf
              c := 255 
          if c == "h"                                   ' html so discard
             repeat 4
               ILI9325.readrambyte(pointer++)
             c := 255
          if c == "c"                                   ' code
            repeat 4
              ILI9325.readrambyte(pointer++)        ' discard the ode>
            ILILoadfont(string("Cour16.ifn"))  
            c:= 255 
          if c == "i"                                   ' italic
             c := ILI9325.readrambyte(pointer++)          ' get next character
            if c == ">"                                 ' it was i for italic
              ILILoadfont(string("Times16i.ifn"))
              c := 255
            if c == "m"                                 ' img so load a picture
              repeat until c == 34                      ' read in until the quote
                c := ILI9325.readrambyte(pointer++)
              str.builtstring(true)                     ' clear the string
              repeat 
                c:=ILI9325.readrambyte(pointer++)
                if c <> 34
                  str.buildstring(c)                           ' get the name of the file and build a string
              while c <> 34
              ' file is 128x98
               fat.openfile(str.builtstring(false),"R")                ' open the picture 128x98
               filesize := fat.filesize                        ' divide by 3 for the number of pixels
               blocks := (filesize / 3) /128                   ' number of 128 byte pixel blocks                 
               ramaddress := 0
               repeat blocks
                 fat.readdata(@sdbuffer,384)                       ' 128 pixels, 3 bytes per pixel
                 ILI9325.docmd("E",@sdbuffer,@buffer2,128)         ' convert to 2 byte .ili format, len is number of pixels
                 ILI9325.docmd("A",@buffer2,ramaddress,256)        ' A must start on 0,256,512,768 etc move 256 bytes to external ram (must be 256)
                 ramaddress += 256                                 ' add 256
               fat.closefile
               ILI9325.draw(0+10,cury,127+10,cury+blocks-1)       ' set up the area of the screen to draw in
               ILI9325.docmd("C",0,0,blocks << 8)                       ' move bytes from ram out to the display
               curx :=0
               cury += blocks   ' need to edit this for other file sizes - only works for 128 wide at the moment
              repeat
                c:=ILI9325.readrambyte(pointer++)
              until c == ">"          
             c :=255
             

             
                
          if c == "/"                                   ' cancel character
            c := ILI9325.readrambyte(pointer++)         ' read in next character after /
            if c == "b"                                 ' could be cancel big or cancel b bold
              c := ILI9325.readrambyte(pointer++)       ' get next character
              if c == ">"
                 ILILoadfont(string("Times16.ifn"))       ' must be cancel bold so reload normal font
                 c := 255
              if c == "o"                               ' cancel body
                repeat 3
                  ILI9325.readrambyte(pointer++)        ' read and discard dy>
                c :=255   
              if c == "i"                               ' cancel big
                repeat 2
                  ILI9325.readrambyte(pointer++)  
                ILILoadfont(string("Times16.ifn"))         '  normal font
                c := 255   
            if c == "p"                                 ' cancel a paragraph
               ILI9325.readrambyte(pointer++)           ' read in ">" and discard
               ILIcrlf                                     ' new paragraph
               c := 255
            if c == "i"                                 ' cancel italic
              ILI9325.readrambyte(pointer++)       ' get next character
              ILILoadfont(string("Times16.ifn"))       ' must be cancel bold so reload normal font
              c := 255
            if c == "c"                                 ' cancel code
              repeat 4
                 ILI9325.readrambyte(pointer++)
              ILILoadfont(string("Times16.ifn"))       ' must be cancel code so reload normal font    
              c := 255 
           if c =="h"
             c := 254                                   ' /html so finish displaying and end the routine
               
      if c == 255                                       ' exit character so read the next one
         c := ILI9325.readrambyte(pointer++)           ' read in next character ready to display     
      if (c=>32) and (c=< 126) and (c <> "<")
          curx := ILIChar(c,curx,cury)                   ' only print if a valid ascii character

    repeat

   
     
PUB NewFont(strpointer)
    ILIloadfont(strpointer)
    ' only call this routine with the cursor curx at zero
    ClearFontBackground(curx,cury,screenwidth,cury+FontHeight)             ' clear an area the same color as the font background  

PUB LoadFile(strpointer) | filesize,ramaddress ' reads a file in lineoftext1 into the external ram
    fat.openfile(strpointer,"R")            ' open a file
    filesize := fat.filesize
    ramaddress := RamText                               ' store text here
    repeat (filesize >>8 ) + 1                          ' so works with 256 byte blocks
      fat.readdata(@sdbuffer,256)                       ' get 256 bytes from sd card
      ILI9325.docmd("A",@sdbuffer,ramaddress,256)       ' A must start on 0,256,512,768 etc move data from hub to external ram 256 bytes at a time    
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile   
    return filesize
{ 
PUB PrintStringCR(pstring)' sends pstring to the vga and serial port
     PrintString(pstring)                               ' print the string  
     crlf

PUB PrintString(pstring)' sends pstring to the vga and serial port with no CRLF
    if (_TV_Enabled)
      text.str(pstring)
'    if (_VGA_Enabled)  
'      vt100.str(pstring)                                 ' send to vga screen
'    if (_Serial_Enabled)   
'    sio.str(0,pstring)                                 ' send to com port
'    if (_lcd_Enabled)
'       lcd.str(pstring)                                   ' send to 20x4 lCD display

PUB PrintChar(c) ' sends c to vga and serial port
    if (_TV_Enabled)
      text.out(c)
'    if (_VGA_Enabled)
'      vt100.out(c)                                        ' send to vga
'    if (_Serial_Enabled)
'      sio.tx(0,c)                                         ' send to com port
'    if (_lcd_Enabled)
'      LCD.out(c)                                          ' send to LCD display      
PUB CRLF
    PrintChar(13)                                        ' send to vga
    PrintChar(10)                                        ' send to vga

PUB PrintDecimal(number)                                ' print the decimal value, useful for debugging
     PrintString(str.integerToDecimal(number, 10))
     crlf

PUB PrintHex(number)
    Printstring(str.integertohexadecimal(number,8)) ' print the hex long value on the TV
}
DAT
sdbuffer        byte    $0[512]  ' 512 byte buffer for sd card interface
buffer2         byte    $0[512]  ' 512 general purpose hub buffer